@@ -3,18 +3,10 @@ kHPOGridEnvPath = 'HPOGRID_BASE_PATH'
 kProjectConfigNameJson = 'project_config.json'
 kProjectConfigNameYaml = 'project_config.yaml'
 
-kModelConfigName = 'model_config.json'
-kSearchSpaceConfigName = 'search_space.json'
-kHPOConfigName = 'hpo_config.json'
-kGridConfigName	 = 'grid_config.json'
-
-kConfig2FileMap = {
-	'project': kProjectConfigNameJson,
-	'hpo': kHPOConfigName,
-	'search_space': kSearchSpaceConfigName,
-	'model': kModelConfigName,
-	'grid': kGridConfigName
-}
+kiDDSConfigName = 'idds_config.json'
+kiDDSSearchSpaceName = 'search_space.json'
+kiDDSHPinput = 'input_hyperparameters.json'
+kiDDSHPoutput = 'output.json'
 
 kAlgoMap = {
     'random': 'tune',
@@ -33,6 +25,7 @@ kDefaultMode = 'min'
 kDefaultScheduler = 'asynchyperband'
 kDefaultLogDir = './log'
 kDefaultTrials = 100
+kDefaultMaxConcurrent = 3
 kDefaultStopping = '{"training_iteration": 1}'
 kDefaultSchedulerParam = '{}'
 kDefaultAlgorithmParam = '{}'
