@@ -80,21 +80,26 @@ class JobBuilder():
     
     
     def reset(self):
-        self._algorithm = None
-        self.algorithm_param = {}
-        self.metric = None
-        self._mode = None
-        self._scheduler = None
-        self.scheduler_param = {}
+        self.project_name = None
+        # model configuration
         self.model_script = None
         self.model_name = None
-        self.model_param = {}
-        self.num_trials = None
+        self.model_param = kDefaultModelParam  
+        # search space
+        self.search_space = {}
+        # hpo configuration
+        self._algorithm = None
+        self.metric = None
+        self._mode = None
+        self._scheduler = None  
+        self.algorithm_param = kDefaultAlgorithmParam        
+        self.scheduler_param = kDefaultSchedulerParam
+        self.num_trials = kDefaultTrials
+        self.max_concurrent = kDefaultMaxConcurrent
         self.verbose = False
         self.log_dir = kDefaultLogDir
-        self.project_name = None
-        self.stop = {}
-        self.search_space = {}
+        self.stop = kDefaultStopping
+        # parameters for logging
         self._df = None
         self._start_datetime = None
         self._end_datetime = None
@@ -102,6 +107,7 @@ class JobBuilder():
         self._best_config = None
         self._hyperparameters = {}
         self._start_timestamp = None
+        self.idds_job = False
 
         
     @staticmethod
@@ -146,26 +152,31 @@ class JobBuilder():
             raise ValueError('Unrecognized search algorithm: {}'.format(name))
 
     @staticmethod
-    def get_algorithm(name, metric, mode, base_search_space, **args):
+    def get_algorithm(name, metric, mode, base_search_space, max_concurrent=None, **args):
         if name == 'ax':
             from hpogrid.algorithm.ax_algorithm import AxAlgoWrapper
-            return AxAlgoWrapper().create(metric, mode, base_search_space, **args)
+            algorithm = AxAlgoWrapper().create(metric, mode, base_search_space, **args)
         elif name == 'bohb':
             from hpogrid.algorithm.bohb_algorithm import BOHBAlgoWrapper
-            return BOHBAlgoWrapper().create(metric, mode, base_search_space, **args)
+            algorithm = BOHBAlgoWrapper().create(metric, mode, base_search_space, **args)
         elif name == 'hyperopt':
             from hpogrid.algorithm.hyperopt_algorithm import HyperOptAlgoWrapper
-            return HyperOptAlgoWrapper().create(metric, mode, base_search_space, **args)
+            algorithm = HyperOptAlgoWrapper().create(metric, mode, base_search_space, **args)
         elif name == 'skopt':
             from hpogrid.algorithm.skopt_algorithm import SkOptAlgoWrapper
-            return SkOptAlgoWrapper().create(metric, mode, base_search_space, **args)
+            algorithm = SkOptAlgoWrapper().create(metric, mode, base_search_space, **args)
         elif name == 'nevergrad':
             from hpogrid.algorithm.nevergrad_algorithm import NeverGradAlgoWrapper
-            return NeverGradAlgoWrapper().create(metric, mode, base_search_space, **args)
+            algorithm = NeverGradAlgoWrapper().create(metric, mode, base_search_space, **args)
         elif name == 'tune':
-            return None
+            algorithm = None
         else:
             raise ValueError('Unrecognized search algorithm: {}'.format(name))
+        # limit max concurrency
+        if algorithm and max_concurrent:
+            from ray.tune.suggest import ConcurrencyLimiter
+            algorithm = ConcurrencyLimiter(algorithm, max_concurrent=max_concurrent) 
+        return algorithm
 
     @staticmethod
     def get_model(script_name, model_name):
@@ -179,7 +190,7 @@ class JobBuilder():
                 'from training script: {}.py'.format(model_name, script_name_noext))
         return model
 
-    def save_metadata(self, df) -> None:
+    def create_metadata(self, df) -> Dict:
 
         summary = {
             'project_name' : self.project_name,
@@ -203,8 +214,15 @@ class JobBuilder():
         df = df.filter(cols_to_save, axis=1).transpose()
         summary['result'] = df.to_dict()
 
-        with open(kGridSiteMetadataFileName,'w') as output:
-            json.dump(summary, output, cls=helper.NpEncoder)
+        return summary
+    
+    def create_idds_output(self, summary):
+        idds_output = {}
+        idds_output['status'] = 0
+        loss = summary['result'][0][summary['metric']]
+        idds_output['loss'] = loss
+        idds_output['message'] = ''
+        return idds_output
 
     @staticmethod
     def get_resource_info() -> Dict:
@@ -239,7 +257,9 @@ class JobBuilder():
             self.metric,
             self.mode,
             self.search_space,
+            self.max_concurrent,
             **self.algorithm_param)
+        
     
         scheduler = JobBuilder.get_scheduler(
             self.scheduler,
@@ -288,30 +308,90 @@ class JobBuilder():
 
         df = analysis.dataframe()
 
-        self.save_metadata(df)
+        metadata = self.create_metadata(df)
+        
+        # save metadata
+        with open(kGridSiteMetadataFileName,'w') as output:
+            json.dump(metadata, output, cls=helper.NpEncoder)
+        
+        # save idds output
+        if self.idds_job:
+            idds_output = self.create_idds_output(metadata)
+            idds_output_path = os.path.join(kWorkDir,kiDDSHPoutput)
+            with open(idds_output_path, 'w') as idds_output_file:
+                json.dump(idds_output, idds_output_file, cls=helper.NpEncoder)
 
+        
     @classmethod
-    def from_project(cls, proj_name:str):
+    def from_project(cls, proj_name:str, idds_job=False):
         
         config = helper.get_project_config(proj_name)
         
         helper.set_script_path(proj_name)
         
+        if idds_job:
+            input_point_path = os.path.join(kWorkDir, kiDDSHPinput)
+            with open(input_point_path,'r') as input_point_file:
+                input_point = json.load(input_point_file)
+            return JobBuilder.from_config_for_idds(config, input_point)
+        
         return JobBuilder.from_config(config)
     
     
+    
     @classmethod
     def from_json(cls, fname:str):
         with open(fname, 'r') as config_file:
             config = json.load(config_file)       
         return JobBuilder.from_config(config)
     
+    @classmethod
+    def from_config_for_idds(cls, config:Dict, input_point:Dict):
+        job = JobBuilder()
+        
+        job.project_name = config['project_name']
+        
+        # Load search space
+        job.search_space = {}
+        
+        # Load hpo configuration
+        job.algorithm = 'random'
+        job.metric = config['hpo_config']['metric']
+        job.mode = config['hpo_config']['mode']
+        job.scheduler = None
+        job.scheduler_param = {}
+        job.algorithm_param = {}
+        job.num_trials = 1
+        job.max_concurrent = 1
+        job.verbose = False
+        job.log_dir = config['hpo_config']['log_dir']
+        job.stop = config['hpo_config']['stop']
+        
+        # Load model configuration
+        job.model_script = config['model_config']['script']
+        job.model_name = config['model_config']['model']
+        job.model_param = config['model_config']['param']
+        
+        if set(input_point.keys()) != set(config['search_space'].keys()):
+            raise ValueError('hyperparameters given by input search point '
+                             'are inconsistent with search space definition')
+        
+        job.model_param.update(input_point)
+        job._hyperparameters = list(input_point.keys())
+        job.idds_job = True
+    
+        return job
+       
+    
     @classmethod
     def from_config(cls, config:Dict):
         job = JobBuilder()
         
         job.project_name = config['project_name']
         
+        # Load search space
+        job.search_space = config['search_space']
+        
         # Load hpo configuration
         job.algorithm = config['hpo_config']['algorithm']
         job.metric = config['hpo_config']['metric']
@@ -320,6 +400,7 @@ class JobBuilder():
         job.scheduler_param = config['hpo_config']['scheduler_param']
         job.algorithm_param = config['hpo_config']['algorithm_param']
         job.num_trials = config['hpo_config']['num_trials']
+        job.max_concurrent = config['hpo_config']['max_concurrent']
         job.verbose = config['hpo_config']['verbose']
         job.log_dir = config['hpo_config']['log_dir']
         job.stop = config['hpo_config']['stop']
@@ -328,29 +409,26 @@ class JobBuilder():
         job.model_script = config['model_config']['script']
         job.model_name = config['model_config']['model']
         job.model_param = config['model_config']['param']
-        
-        # Load search space
-        job.search_space = config['search_space']
 
         job._hyperparameters = list(job.search_space.keys())
     
         return job
        
-def local_job_parser():
+def local_job_parser(idds_job=False):
     parser = argparse.ArgumentParser(description='run hyperparameter optimization '
                                     'for a user defined machine learning project')     
     parser.add_argument('proj_name', help='Name of the project on which '
                         'hyperparameter optimization is run')    
     args = parser.parse_args(sys.argv[2:])
 
-    job = JobBuilder.from_project(args.proj_name)
+    job = JobBuilder.from_project(args.proj_name, idds_job=idds_job)
     job.run()
             
-def grid_job_parser():
+def grid_job_parser(idds_job=False):
     try:
         extracted_files = helper.extract_tarball(kDataDir, kDataDir)
         helper.grid_site_setup()
-        local_job_parser()
+        local_job_parser(idds_job=idds_job)
     finally:
         helper.remove_files(extracted_files)
             
