@@ -166,8 +166,8 @@ class ConfigurationBase():
         table = stylus.create_table(config_list, self.list_columns)
         print(table)
 
-    def show(self, name, yaml_depth=None):
+    def show(self, name):
         config = self.load_config(name)
-        config_display = stylus.create_formatted_dict(config, self.show_columns, indexed=False,
-                                           yaml_depth=yaml_depth)
-        print(config_display)
+        formatted_str = stylus.create_table(config, self.show_columns, indexed=False,
+                                           wrap_nested_dict=True)
+        print(formatted_str)
