@@ -7,6 +7,7 @@ import shutil
 from distutils import dir_util
 from datetime import datetime
 from json import JSONDecodeError
+from pdb import set_trace
 
 from hpogrid.utils import stylus, helper
 from hpogrid.components.defaults import *
@@ -105,9 +106,11 @@ class ProjectConfiguration(ConfigurationBase):
         # create project directories
         scripts_dir = os.path.join(proj_path, 'scripts')
         config_dir = os.path.join(proj_path, 'config')
+        idds_dir = os.path.join(proj_path, 'idds')
         os.makedirs(proj_path, exist_ok=True)        
         os.makedirs(scripts_dir, exist_ok=True)
         os.makedirs(config_dir, exist_ok=True)
+        os.makedirs(idds_dir, exist_ok=True)
 
         # copy training scripts to the project directory
         if ('scripts_path' in config) and (config['scripts_path'] is not None):
@@ -131,6 +134,8 @@ class ProjectConfiguration(ConfigurationBase):
     
         project_config_path_json = os.path.join(config_dir, kProjectConfigNameJson)
         project_config_path_yaml = os.path.join(config_dir, kProjectConfigNameYaml)
+        idds_config_path = os.path.join(idds_dir, kiDDSConfigName)
+        idds_search_space_path = os.path.join(idds_dir, kiDDSSearchSpaceName)
         
         with open(project_config_path_json,'w') as proj_config_file:
             json.dump(project_config, proj_config_file, indent=2)
@@ -138,7 +143,16 @@ class ProjectConfiguration(ConfigurationBase):
         with open(project_config_path_yaml,'w') as proj_config_file:
             yaml.dump(project_config, proj_config_file, default_flow_style=False, sort_keys=False)
             print('INFO: Created project configuration: {}'.format(project_config_path_yaml))
-            
+        
+        # save configurations for idds workflow
+        idds_config = self.to_idds_config(project_config)
+        with open(idds_config_path, 'w') as idds_config_file:
+            json.dump(idds_config, idds_config_file, indent=2)
+            print('INFO: Created iDDS configuration: {}'.format(idds_config_path))
+        search_space = project_config['search_space']
+        with open(idds_search_space_path, 'w') as idds_search_space_file:
+            json.dump(search_space, idds_search_space_file)
+            print('INFO: Created iDDS search space file: {}'.format(idds_search_space_path))
         '''
         for key in kConfigList:
             if (key in config) and (config[key] is not None):
@@ -154,7 +168,39 @@ class ProjectConfiguration(ConfigurationBase):
         with open(project_config_path,'w') as proj_config_file:
             json.dump(project_config, proj_config_file, indent=2)
         '''
+        
+    # subject to updates
+    @staticmethod
+    def to_idds_config(config):
+        project_name = config['project_name']
+        evaluation_container = config['grid_config']['container']
+        library = config['hpo_config']['algorithm']
+        method = config['hpo_config']['algorithm_param'].get('method', None)
+        in_ds = config['grid_config']['inDS']
+        
+        idds_config = {}
+        generator_options = {}
+        generator_options['n_point'] = '%NUM_POINTS'
+        generator_options['max_point'] = '%MAX_POINTS'
+        generator_options['infile'] = os.path.join(kWorkDir, '%IN')
+        generator_options['outfile'] = os.path.join(kWorkDir, '%OUT')
+        generator_options['library'] = library
+        if method:
+            generator_options['method'] = method
+        generator_cmd = 'hpogrid generate ' + stylus.join_options(generator_options)
+        
+        idds_config['steeringExec'] = ('run --rm -v \"$(pwd)\":{workdir} {container} '
+        '/bin/bash -c \"{cmd}\"').format(workdir=kWorkDir, container=kDefaultContainer, cmd=generator_cmd)
+        idds_config['evaluationExec'] = 'hpogrid idds_run {project_name}'.format(project_name=project_name)
+        idds_config['evaluationContainer'] = evaluation_container
+        idds_config['evaluationInput'] = kiDDSHPinput
+        idds_config['evaluationOutput'] = kiDDSHPoutput
+        idds_config['evaluationMeta'] = kGridSiteMetadataFileName
+        idds_config['nParallelEvaluation'] = config['hpo_config']['max_concurrent']
+        idds_config['trainingDS'] = in_ds        
 
+        return idds_config
+    
     def get_config_list(self, expr=None):
         project_list = [ s for s in super().get_config_list() if s is not 'backup']
         return project_list
@@ -173,4 +219,4 @@ class ProjectConfiguration(ConfigurationBase):
             config = helper.get_project_config(name)
             print(yaml.dump(config, allow_unicode=True, default_flow_style=False, sort_keys=False))
         else:
-            print('ERROR: Project {} does not exist.'.format(name))
+            print('ERROR: Project {} does not exist.'.format(name))
\ No newline at end of file
