@@ -1,6 +1,7 @@
 import os
 import sys
 import json
+import shutil
 import tarfile
 import argparse
 import numpy as np
@@ -54,7 +55,7 @@ def get_project_path(proj_name):
         base_path = get_base_path()
         proj_path = os.path.join(base_path, 'project', proj_name)
     if not os.path.exists(proj_path):
-        raise RunTimeError('Project "{}" not found.'.format(proj_name))
+        raise RuntimeError('Project "{}" not found.'.format(proj_name))
     return proj_path
 
 def get_project_config(proj_name):
