@@ -22,6 +22,9 @@ class ColorCode():
     UNDERLINE = '\033[4m'
     BLINK = '\033[5m'
 
+def join_options(options):
+    return ' '.join(["--{} {}".format(key,value) for (key,value) in options.items()])
+
 def create_table(data, columns=None, indexed=True, transpose=False,
     tableformat=kDefaultTableStyle, stralign=kDefaultStrAlign,
     wrap_nested_dict=False):
